<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl='http://www.w3.org/1999/XSL/Transform' xmlns:SInWs="Sum-In_Words.uri" version='1.0' exclude-result-prefixes='SInWs'>
	<xsl:output method='xml'/>
 
 <xsl:template name="SplitTextBySpace">
    <xsl:param name="Text"/>
    <xsl:param name="Len"/>
    <xsl:choose>
      <xsl:when test="string-length($Text)&lt;$Len">
        <xsl:value-of select="$Text"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="sSubstring">
          <xsl:choose>
            <xsl:when test="starts-with($Text, '&#xA;')">
              <xsl:value-of select="string(' ')"/>
            </xsl:when>
            <xsl:when test="contains($Text, '&#xA;')">
              <xsl:value-of select="concat(substring-before($Text,'&#xA;'),'&#xA;')"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="substring($Text,1, $Len)"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:variable name="TextBefore" select="$sSubstring"/>
        <xsl:variable name="TextBeforeLen" select="string-length($TextBefore)"/>
        <xsl:if test="$TextBeforeLen &lt;= $Len">
          <xsl:variable name="TextAfter" select="substring($Text,$TextBeforeLen+1)"/>
          <xsl:value-of select="$TextBefore"/>
          <xsl:if test="($Len - $TextBeforeLen) >0 and contains($TextAfter, '&#xA;')">
            <xsl:variable name="sResultAfter">
              <xsl:call-template name="SplitTextBySpace">
                <xsl:with-param name="Text" select="$TextAfter"/>
                <xsl:with-param name="Len" select="$Len - $TextBeforeLen"/>
              </xsl:call-template>
            </xsl:variable>
            <xsl:value-of select="$sResultAfter"/>
          </xsl:if>
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

 <xsl:variable name='Self' select='document("")'/>
	<xsl:template name='SumInWords'>
		<xsl:param name="Sum"/>
		<xsl:call-template name='RecurseSumWords'>
			<xsl:with-param name='Value' select='$Sum'/>
		</xsl:call-template>
	</xsl:template>

	<xsl:template name='RecurseSumWords'>
		<xsl:param name="Value"/>
		<xsl:choose>
			<xsl:when test='($Value div 1000000) - (($Value mod 1000000) div 1000000) > 0'>
				<xsl:call-template name='DecodeInteger'>
					<xsl:with-param name='Value' select='($Value div 1000000) - (($Value mod 1000000) div 1000000)'/>
					<xsl:with-param name='controlbad' select='0'/>
				</xsl:call-template>
				<xsl:call-template name='GetWordType'>
					<xsl:with-param name='Value' select='($Value div 1000000) - (($Value mod 1000000) div 1000000)'/>
					<xsl:with-param name='tagName' select="'SInWs:Billion'"/>
				</xsl:call-template>
				<xsl:call-template name='RecurseSumWords'>
					<xsl:with-param name='Value' select='$Value mod 1000000'/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test='($Value div 1000) - (($Value mod 1000) div 1000) > 0'>
				<xsl:call-template name='DecodeInteger'>
					<xsl:with-param name='Value' select='($Value div 1000) - (($Value mod 1000) div 1000)'/>
					<xsl:with-param name='controlbad' select='1'/>
				</xsl:call-template>
				<xsl:call-template name='GetWordType'>
					<xsl:with-param name='Value' select='($Value div 1000) - (($Value mod 1000) div 1000)'/>
					<xsl:with-param name='tagName' select="'SInWs:Thousand'"/>
				</xsl:call-template>
				<xsl:call-template name='RecurseSumWords'>
					<xsl:with-param name='Value' select='$Value mod 1000'/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test='$Value != 0'>
				<xsl:call-template name='DecodeInteger'>
					<xsl:with-param name='Value' select='$Value'/>
					<xsl:with-param name='controlbad' select='0'/>
				</xsl:call-template>
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template name='DecodeInteger'>
		<xsl:param name='Value' select="''"/>
		<xsl:param name='controlbad' select="''"/>
		<xsl:variable name='Value100' select='$Value mod 100'/>
		<xsl:if test='($Value div 100) - ($Value100 div 100) != 0'>
			<xsl:call-template name='GetWordData'>
				<xsl:with-param name='Pos' select='27 + (($Value div 100) - ($Value100 div 100))'/>
				<xsl:with-param name='tagName' select="'SInWs:Lit'"/>
			</xsl:call-template>
		</xsl:if>
		<xsl:choose>
			<xsl:when test='$Value100 > 20'>
				<xsl:call-template name='GetWordData'>
					<xsl:with-param name='Pos' select='18 + (($Value100 div 10) - (($Value100 mod 10) div 10))'/>
					<xsl:with-param name='tagName' select="'SInWs:Lit'"/>
				</xsl:call-template>
				<xsl:if test='$Value100 mod 10 != 0'>
					<xsl:call-template name='GetSmallWord'>
						<xsl:with-param name='Value' select='$Value100 mod 10'/>
						<xsl:with-param name='controlbad' select='$controlbad'/>
					</xsl:call-template>
				</xsl:if>
			</xsl:when>
			<xsl:when test='$Value100 != 0'>
				<xsl:call-template name='GetSmallWord'>
					<xsl:with-param name='Value' select='$Value100'/>
					<xsl:with-param name='controlbad' select='$controlbad'/>
				</xsl:call-template>
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template name='GetSmallWord'>
		<xsl:param name='Value' select="''"/>
		<xsl:param name='controlbad' select="''"/>
		<xsl:choose>
			<xsl:when test='($controlbad = 1) and ($Value &lt; 3)'>
				<xsl:call-template name='GetWordData'>
					<xsl:with-param name='Pos' select='$Value'/>
					<xsl:with-param name='tagName' select="'SInWs:BadWord'"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name='GetWordData'>
					<xsl:with-param name='Pos' select='$Value'/>
					<xsl:with-param name='tagName' select="'SInWs:Lit'"/>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name='GetWordType'>
		<xsl:param name='Value' select="''"/>
		<xsl:param name='tagName' select="''"/>
		<xsl:choose>
			<xsl:when test='($Value mod 100) > 20'>
				<xsl:call-template name='GetInnerType'>
					<xsl:with-param name='Value' select='($Value mod 100) mod 10'/>
					<xsl:with-param name='tagName' select='$tagName'/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name='GetInnerType'>
					<xsl:with-param name='Value' select='$Value mod 100'/>
					<xsl:with-param name='tagName' select='$tagName'/>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name='GetInnerType'>
		<xsl:param name='Value' select="''"/>
		<xsl:param name='tagName' select="''"/>
		<xsl:choose>
			<xsl:when test='$Value = 1'>
				<xsl:call-template name='GetWordData'>
					<xsl:with-param name='Pos' select='1'/>
					<xsl:with-param name='tagName' select='$tagName'/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test='$Value > 1 and $Value &lt; 5'>
				<xsl:call-template name='GetWordData'>
					<xsl:with-param name='Pos' select='2'/>
					<xsl:with-param name='tagName' select='$tagName'/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name='GetWordData'>
					<xsl:with-param name='Pos' select='3'/>
					<xsl:with-param name='tagName' select='$tagName'/>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
		<xsl:template name="GetWordData">
		<xsl:param name="Pos"/>
		<xsl:param name="tagName"/>
		<xsl:for-each select="$Self//SInWs:*[name()=$tagName]">
			<xsl:for-each select="word[position()=format-number($Pos,'#')]">
					<xsl:value-of select="."/>
					<xsl:text> </xsl:text>
			</xsl:for-each>
		</xsl:for-each>
	</xsl:template>
	
	<SInWs:Lit>
		<word>ОДИН</word>
		<word>ДВА</word>
		<word>ТРИ</word>
		<word>ЧЕТЫРЕ</word>
		<word>ПЯТЬ</word>
		<word>ШЕСТЬ</word>
		<word>СЕМЬ</word>
		<word>ВОСЕМЬ</word>
		<word>ДЕВЯТЬ</word>
		<word>ДЕСЯТЬ</word>
		<word>ОДИННАДЦАТЬ</word>
		<word>ДВЕНАДЦАТЬ</word>
		<word>ТРИНАДЦАТЬ</word>
		<word>ЧЕТЫРНАДЦАТЬ</word>
		<word>ПЯТНАДЦАТЬ</word>
		<word>ШЕСТНАДЦАТЬ</word>
		<word>СЕМНАДЦАТЬ</word>
		<word>ВОСЕМНАДЦАТЬ</word>
		<word>ДЕВЯТНАДЦАТЬ</word>
		<word>ДВАДЦАТЬ</word>
		<word>ТРИДЦАТЬ</word>
		<word>СОРОК</word>
		<word>ПЯТЬДЕСЯТ</word>
		<word>ШЕСТЬДЕСЯТ</word>
		<word>СЕМЬДЕСЯТ</word>
		<word>ВОСЕМЬДЕСЯТ</word>
		<word>ДЕВЯНОСТО</word>
		<word>СТО</word>
		<word>ДВЕСТИ</word>
		<word>ТРИСТА</word>
		<word>ЧЕТЫРЕСТА</word>
		<word>ПЯТЬСОТ</word>
		<word>ШЕСТЬСОТ</word>
		<word>СЕМЬСОТ</word>
		<word>ВОСЕМЬСОТ</word>
		<word>ДЕВЯТЬСОТ</word>
	</SInWs:Lit>
	<SInWs:Billion>
		<word>МИЛЛИОН</word>
		<word>МИЛЛИОНА</word>
		<word>МИЛЛИОНОВ</word>
	</SInWs:Billion>
	<SInWs:Thousand>
		<word>ТЫСЯЧА</word>
		<word>ТЫСЯЧИ</word>
		<word>ТЫСЯЧ</word>
	</SInWs:Thousand>
	<SInWs:BadWord>
		<word>ОДНА</word>
		<word>ДВЕ</word>
	</SInWs:BadWord>

</xsl:stylesheet>
